/**
 * foghat.c - Simple PostgreSQL program to query database and extract Foghat tunes.
 *
 * Author: Randy L. Pearson
 * Date:   8 Jul 2004
 */
 
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <termios.h>
#include <errno.h>
#include <libpq-fe.h>

/* Function prototypes */

void command_line( int argc, char *argv[] );
void run_query();
void get_password();
void usage();

/* Default values that can be overridden by command line */

const char *dbserver = "localhost";
const char *username = "randy";
char password[255];
const char *database = "music";
const char *dbport = "5432";

/* Error codes we might return */
#if !defined(NO_ERROR)
#define NO_ERROR	0
#endif
#define INVALID_OPTION	1
#define NO_CONNECTION	2
#define SQL_ERROR		3
#define TTY_ERROR		4

/* Global variables */

PGconn *connection = NULL;

/*********************************
 * main() - start of program
 *********************************/
 
int main( int argc, char *argv[] )
{
	command_line( argc, argv );
	
	connection = PQsetdbLogin( dbserver, dbport, 0, 0, database, username, password );
	if( PQstatus(connection) != CONNECTION_OK )
	{
		fprintf( stderr, "PQsetdbLogin() failed.\n" );
		fprintf( stderr, "Error: %s\n", PQerrorMessage(connection) );
		PQfinish(connection);
		exit(NO_CONNECTION);
	}
	
	run_query();
	
	PQfinish(connection);
}

/*********************************
 * command_line() - process command
 *                  line arguments
 *********************************/
 
void command_line( int argc, char *argv[] )
{
	int option;
	
	while( (option = getopt(argc, argv, "hs:u:d:p::")) != -1 )
	{
		switch( option )
		{
		case 'h':
			usage();
			exit(NO_ERROR);
			break;
		case 's':
			dbserver = optarg;
			break;
		case 'u':
			username = optarg;
			break;
		case 'd':
			database = optarg;
			break;
		case 'p':
			if( optarg )
			{
				strncpy( password, optarg, 255 );
				password[254] = 0;
			}
			else
				get_password();
			break;
		case '?':
		case ':':
			usage();
			exit(INVALID_OPTION);
			break;
		}
	}
}

/*********************************
 * run_query() - Run PostgreSQL query
 *********************************/
 
void run_query()
{
	char query[2048];
	PGresult *result;
	int row;
	
	strcpy( query, "SELECT artist,album,song,genre FROM song "
					"JOIN album ON song.album_id = album.album_id "
					"JOIN artist ON song.artist_id = artist.artist_id "
					"WHERE artist.artist = 'Foghat' "
					"ORDER BY album,song" );
	
	result = PQexec( connection, query );
	
	if( !result || PQresultStatus(result) != PGRES_TUPLES_OK )
	{
		fprintf( stderr, "Query Error: %s\n", PQresStatus(PQresultStatus(result)) );
		fprintf( stderr, "Details: %s\n", PQresultErrorMessage(result) );
		PQclear(result);
		return;
	}
	
	for( row = 0; row < PQntuples(result); row++ )
	{
		printf( "%s %s %s %s\n",
			PQgetvalue(result, row, 0),
			PQgetvalue(result, row, 1),
			PQgetvalue(result, row, 2),
			PQgetvalue(result, row, 3) );
	}
	
	PQclear(result);
}

/*********************************
 * get_password() - password input
 *********************************/
 
void get_password()
{
	struct termios old_settings, new_settings;
	FILE * tty;
	
	tty = fopen( "/dev/tty", "rw" );
	if( tty == 0 )
	{
		fprintf( stderr, "Unable to open terminal device: %s\n", strerror(errno) );
		exit(TTY_ERROR);
	}
	
	tcgetattr( fileno(tty), &old_settings );
	
	new_settings = old_settings;
	new_settings.c_lflag &= ~ECHO;
	
	printf( "Enter password: " );
	
	if( tcsetattr( fileno(tty), TCSAFLUSH, &new_settings) != 0 )
	{
		fprintf( stderr, "Could not configure terminal: %s\n", strerror(errno) );
		exit(TTY_ERROR);
	}
	
	fgets( password, 255, tty );
	tcsetattr( fileno(tty), TCSANOW, &old_settings );
	fclose(tty);
}

/*********************************
 * usage() - show command help
 *********************************/
 
void usage()
{
	fprintf( stderr, "Usage: foghat [OPTION]...\n\n" );
	fprintf( stderr, "Options:\n" );
	fprintf( stderr, "  -h            Displays this help message\n" );
	fprintf( stderr, "  -s server     Use the named database server\n" );
	fprintf( stderr, "                  (default: %s)\n", dbserver );
	fprintf( stderr, "  -u username   Use the supplied user name\n" );
	fprintf( stderr, "                  (default: %s)\n", username );
	fprintf( stderr, "  -d database   Use the supplied database\n" );
	fprintf( stderr, "                  (default: %s)\n", database );
	fprintf( stderr, "  -p [password] Use the supplied password\n" );
	fprintf( stderr, "                   Prompts if no password supplied\n\n" );
	fprintf( stderr, "Report bugs to <rlp@bamafolks.com>.\n" );
}
