/* scandir.c -- Example program showing how to read all the files in the
 *              current directory and use stat() to display details.
 *
 * Author: Randy L. Pearson
 * Date:   17 Jun 2004
 */

#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <dirent.h>
#include <stdio.h>

int recurse = 0;
int long_list = 0;

void showdir( const char *dir );
void recurse_dir( const char * dir );
void show_details( const char *dir, struct dirent *entry );
const char *filemode( mode_t mode );

int main( int argc, char **argv )
{
	int found = 0;
	int i;
	
	/*
	 * Process command line, look for '-h', '-l' or '-r'
	 */
	for( i = 1; i < argc; i++ )
	{
		if( argv[i][0] == '-' )
		{
			if( argv[i][1] == 'l' )
				long_list = 1;
			else if( argv[i][1] == 'r' )
				recurse = 1;
			else if( argv[i][1] == 'h' )
			{
				fprintf( stderr, "Usage: scandir [OPTIONS] [DIR] ...\n\n" );
				fprintf( stderr, "Options:\n" );
				fprintf( stderr, "  -h    Display this text.\n" );
				fprintf( stderr, "  -l    Show long listing.\n" );
				fprintf( stderr, "  -r    Recurse subdirectories.\n" );
				exit(0);
			}
			else
				fprintf( stderr, "Unknown option: %s\nTry \"scandir -h\"\n", argv[i] );
		}
	}
	
	/*
	 * Process command line again, skip options and process directory names
	 */
	for( i = 1; i < argc; i++ )
	{
		if( argv[i][0] != '-' )
		{
			showdir( argv[i] );
			found = 1;
		}
	}
	
	/*
	 * If no directory was listed on command line, show current directory
	 */
	if( !found )
	{
		char curr_dir[NAME_MAX];
		showdir( getcwd( curr_dir, NAME_MAX ) );
	}
	
	exit(0);
}

void showdir( const char * dir )
{
	DIR * pDir = opendir( dir );
	int count = 0;
	struct dirent *entry;

	/* If pointer is NULL, show error and return */	
	if( pDir == NULL )
	{
		perror( dir );
		return;
	}
	
	printf( "Files in directory: %s\n", dir );
	printf( "====================" );
	for( count = 0; count < strlen(dir); count++ )
		printf( "=" );
	printf( "\n" );
	count = 0;
	
	/* Get next filename and loop until NULL */
	while( (entry = readdir( pDir )) != NULL )
	{
		++count;
		
		if( long_list )
			show_details( dir, entry );
		else
			printf( "%s\n", entry->d_name );
			
	}
	
	/* Show summary and close directory */
	printf( "%d files found in directory \"%s\"\n\n", count, dir );
	closedir( pDir );

	/* Handle '-r' option */
	if( recurse )
		recurse_dir( dir );
}

void recurse_dir( const char * dir )
{
	DIR * pDir = opendir( dir );
	struct dirent * entry;
	struct stat file_stat;
	char full_name[NAME_MAX];

	/* If pointer is NULL, just ignore and return */
	if( pDir == NULL )
		return;
		
	while( (entry = readdir( pDir )) != NULL )
	{
		/* Build full pathname by appending filename to directory name */
		strncpy( full_name, dir, NAME_MAX );
		strcat( full_name, "/" );
		strncat( full_name, entry->d_name, NAME_MAX-strlen(full_name) );
	
		/* Read the file status information */
		if( stat( full_name, &file_stat ) == -1 )
			perror( full_name );
		else
		{
			/* If this is a directory, recurse down into it */
			if( S_ISDIR(file_stat.st_mode) )
			{
				/* Ignore "." and ".." entries to prevent endless loops */
				if( strcmp( entry->d_name, "." ) != 0 &&
					strcmp( entry->d_name, ".." ) )
					
					showdir( full_name );
			}
		}
	}
	
	closedir( pDir );
}

void show_details( const char *dir, struct dirent *entry )
{
	struct stat file_stat;
	char full_name[NAME_MAX];
	
	strncpy( full_name, dir, NAME_MAX );
	strcat( full_name, "/" );
	strncat( full_name, entry->d_name, NAME_MAX-strlen(full_name) );
	
	if( stat( full_name, &file_stat ) == -1 )
	{
		perror( full_name );
		return;
	}
	
	printf( "\nFile: %s\n\n", full_name );
	printf( "Size ......... %ld\n",  file_stat.st_size );
	printf( "Permissions .. %s\n",   filemode(file_stat.st_mode) );
	printf( "Hard Links ... %d\n",   file_stat.st_nlink );
	printf( "Device ....... %d\n",   file_stat.st_dev );
	printf( "I-node ....... %d\n",   file_stat.st_ino );
	printf( "User ID ...... %d\n",   file_stat.st_uid );
	printf( "Group ID ..... %d\n",   file_stat.st_gid );
	printf( "Access ....... %s",     ctime(&file_stat.st_atime) );
	printf( "Modified ..... %s",     ctime(&file_stat.st_mtime) );
	printf( "Changed ...... %s\n",   ctime(&file_stat.st_ctime) );
}

const char * filemode( mode_t mode )
{
	static char str_mode[11];
	
	strcpy( str_mode, " ---------" );
	
	if( mode & S_IFSOCK )		str_mode[0] = 'S';
	if( mode & S_IFLNK )		str_mode[0] = 's';
	if( mode & S_IFREG )		str_mode[0] = '-';
	if( mode & S_IFBLK )		str_mode[0] = 'b';
	if( mode & S_IFDIR )		str_mode[0] = 'd';
	if( mode & S_IFCHR )		str_mode[0] = 'c';
	if( mode & S_IFIFO )		str_mode[0] = 'f';
	if( mode & S_ISVTX )		str_mode[0] = 't';
	
	if( mode & S_IRUSR )		str_mode[1] = 'r';
	if( mode & S_IWUSR )		str_mode[2] = 'w';
	if( mode & S_IXUSR )		str_mode[3] = 'x';

	if( mode & S_IRGRP )		str_mode[4] = 'r';
	if( mode & S_IWGRP )		str_mode[5] = 'w';
	if( mode & S_IXGRP )		str_mode[6] = 'x';

	if( mode & S_IROTH )		str_mode[7] = 'r';
	if( mode & S_IWOTH )		str_mode[8] = 'w';
	if( mode & S_IXOTH )		str_mode[9] = 'x';

	if( mode & S_ISUID )		str_mode[3] = 's';
	if( mode & S_ISGID )		str_mode[6] = 's';
	
	return str_mode;
}
