/******
 *
 * smtp-blocker.c:  This file defines the command line handling,
 *                  file opening/closing code and main() function.
 *
 * Author: Randy L. Pearson
 *
 * Revision History -
 *
 * 10 Jun 2004 - Created it.
 *
 ******/
 
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "smtp-blocker.h"

#define NO_ERROR	0
#define USAGE_ERROR	1
#define FILE_ERROR	2

/* Global variables */
FILE *src;
FILE *dest;
int verbose = 0;
static char *input_file = 0;
static char *output_file = 0;
static int append_mode = 0;

/**
 * usage: display usage (help) information to user and exits program.
 **/
 
void usage()
{
	fprintf( stderr, "Usage: smtp-blocker [OPTION]...\n\n" );
	fprintf( stderr, "Options:\n" );
	fprintf( stderr, "  -h, --help         Displays this help message\n" );
	fprintf( stderr, "  -v, --verbose      Displays information while processing\n" );
	fprintf( stderr, "  -i, --input FILE   Reads in data from the named file\n" );
	fprintf( stderr, "                       (default: read stdin)\n" );
	fprintf( stderr, "  -o, --output FILE  Write output to the name file\n" );
	fprintf( stderr, "                       (default: write to stdout)\n" );
	fprintf( stderr, "  -a, --append       Appends output to the file.  Only\n" );
	fprintf( stderr, "                       valid when used with --output.\n\n" );
	fprintf( stderr, "Report bugs to <rlp@bamafolks.com>.\n" );
	
	exit(USAGE_ERROR);
}

/**
 * process_cmd_line: evaluate the command line arguments
 *
 * inputs:
 *
 *   argc - the number of command line arguments
 *   argv - pointer to the command line as an array of strings
 **/

void process_cmd_line(int argc, char **argv)
{
	int opt_char;
	
	while( 1 )
	{
		int optindex = optind ? optind : 1;
		int index = 0;
		
		static struct option long_options[] =
		{
			{ "help",    0, 0, 'h' },
			{ "input",   1, 0, 'i' },
			{ "output",  1, 0, 'o' },
			{ "append",  0, 0, 'a' },
			{ "verbose", 0, 0, 'v' },
			{ 0, 0, 0, 0 }
		};
		
		opt_char = getopt_long (argc, argv, "hi:o:av",
				long_options, &index );
		
		// A -1 means end of option list
		if( opt_char == -1 )
			break;
			
		switch( opt_char )
		{
		case 0:
			printf( "option %s", long_options[index].name );
			if( optarg )
				printf( " with arg %s", optarg );
			printf( "\n" );
			break;
			
		case 'h':
			usage();
			break;
			
		case 'i':
			input_file = optarg;
			break;
			
		case 'o':
			output_file = optarg;
			break;
			
		case 'a':
			append_mode = 1;
			break;
			
		case 'v':
			verbose = 1;
			break;
			
		case '?':
			usage();
			break;
			
		case ':':
			fprintf( stderr, "Missing argument for option: %s\n\n", long_options[index].name );
			usage();
			break;
		}
	}
	
	if( optind < argc)
	{
		fprintf( stderr, "Invalid option(s): " );
		while( optind < argc )
			fprintf( stderr, "%s ", argv[optind++] );
		fprintf( stderr, "\n\n" );
		usage();
	}
}

/**
 * open_files: open input and/or output files as needed
 **/
 
void open_files()
{
	if( input_file )
	{
		if( (src = fopen( input_file, "r" )) == 0 )
		{
			perror( input_file );
			exit(FILE_ERROR);
		}
	}
	else
		src = stdin;
	
	if( output_file )
	{
		if( (dest = fopen( output_file, append_mode ? "a" : "w" )) == 0 )
		{
			perror( output_file );
			exit(FILE_ERROR);
		}
	}
	else
		dest = stdout;
}

int main(int argc, char **argv)
{
	// Step 1: Process command line arguments
	//
	// Usage: stmp-blocker [OPTION]
	//
	// Options:
	//   -h, --help         Displays this help message
	//   -i, --input FILE   Reads in data from the named file
	//                        (default: read stdin)
	//   -o, --output FILE  Write output to the name file
	//                        (default: write to stdout)
	//   -a, --append       Appends output to the file.  Only
	//                        valid when used with --output.
	
	process_cmd_line( argc, argv );
	
	// Step 2: Open input/output files if needed
	
	open_files();
	
	// Step 3: Read input line
	// Step 4: Examine input for SPF block message
	// Step 5: Issue WHOIS lookup against IP address
	// Step 6: Analyze response
	// Step 7: Write out network name and IP range
	// Step 8: Until EOF, goto step 3
	
	process_data();
	
	// Step 9: Close files and exit
	
	if( input_file )
		fclose( src );
		
	if( output_file )
		fclose( dest );
	
	exit(NO_ERROR);
}
