/*
 * tempfiles.c -- Demonstrates generating temporary file names and/or files.
 *
 * Author: Randy L. Pearson
 * Date:   22 Jun 2004
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

/* Function prototype for helper function */
const char * filemode( mode_t mode );

int main()
{
	char tempname[4][L_tmpnam];
	char *temp1, *temp2;
	int i;
	FILE *tempfile;
	int file_handle;
	
	/* Create some temporary file names */
	strcpy( tempname[0], tmpnam(0) );
	tmpnam(tempname[1]);
	strcpy( tempname[2], "/tmp/rlp-XXXXXX" );
	mktemp( tempname[2] );
	strcpy( tempname[3], tempnam(0,"rlp-") );
	
	printf( "Temporary file names -\n" );
	for( i = 0; i < 4; i++ )
		printf( "\ttempname[%d] = %s\n", i, tempname[i] );
		
	/* Why is tmpnam() dangerous?  This is why. */
	temp1 = tmpnam(0);
	temp2 = tmpnam(0);
	
	printf( "\nProblem with tmpnam() function\n" );
	printf( "\tTemp file #1: %s\n", temp1 );
	printf( "\tTemp file #2: %s\n", temp2 );
	
	/* It is safer to use one of these 2 methods */
	tempfile = tmpfile();
	if( tempfile )
	{
		struct stat filestat;
		
		printf( "\nCreated temporary file using tmpfile().\nDetails -\n" );
		fstat( fileno(tempfile), &filestat );
		printf( "\tI-node: %ld\n", filestat.st_ino );
		printf( "\tPermissions: %s\n", filemode(filestat.st_mode) );
		printf( "\tSorry, but there is no easy way to get the filename.\n" );
	
		/* This also deletes the temporary file */	
		fclose( tempfile );
	}
	else
		perror( "tmpfile() failure" );
	
	strcpy( tempname[0], "/tmp/rlp-XXXXXX" );
	file_handle = mkstemp( tempname[0] );
	if( file_handle != -1 )
	{
		struct stat filestat;
		
		printf( "\nCreated temporary file using mkstemp().\nDetails -\n" );
		fstat( file_handle, &filestat );
		printf( "\tI-node: %ld\n", filestat.st_ino );
		printf( "\tPermissions: %s\n", filemode(filestat.st_mode) );
		printf( "\tFilename: %s\n", tempname[0] );
		
		close( file_handle );
		
		/* Must delete the file yourself when using mkstemp()
		 *
		 * NOTE: You may need to copy the filename to a new buffer!
		 */
		if( unlink( tempname[0] ) )
			perror( "Failed to delete file" );
	}
	else
		perror( "mkstemp() failure" );
		
	return 0;
}

const char * filemode( mode_t mode )
{
	static char str_mode[11];
	
	strcpy( str_mode, " ---------" );
	
	if( mode & S_IFSOCK )		str_mode[0] = 'S';
	if( mode & S_IFLNK )		str_mode[0] = 's';
	if( mode & S_IFREG )		str_mode[0] = '-';
	if( mode & S_IFBLK )		str_mode[0] = 'b';
	if( mode & S_IFDIR )		str_mode[0] = 'd';
	if( mode & S_IFCHR )		str_mode[0] = 'c';
	if( mode & S_IFIFO )		str_mode[0] = 'f';
	if( mode & S_ISVTX )		str_mode[0] = 't';
	
	if( mode & S_IRUSR )		str_mode[1] = 'r';
	if( mode & S_IWUSR )		str_mode[2] = 'w';
	if( mode & S_IXUSR )		str_mode[3] = 'x';

	if( mode & S_IRGRP )		str_mode[4] = 'r';
	if( mode & S_IWGRP )		str_mode[5] = 'w';
	if( mode & S_IXGRP )		str_mode[6] = 'x';

	if( mode & S_IROTH )		str_mode[7] = 'r';
	if( mode & S_IWOTH )		str_mode[8] = 'w';
	if( mode & S_IXOTH )		str_mode[9] = 'x';

	if( mode & S_ISUID )		str_mode[3] = 's';
	if( mode & S_ISGID )		str_mode[6] = 's';
	
	return str_mode;
}
